<?php

/**
 * @file
 * TVB administration menu items.
 */

/**
 * Processes Payment Response from Triveneto Bassilichi.
 */
function ubercart_constriv_notify() {

  if (array_key_exists('Error', $_POST)) {
    watchdog('ubercart_constriv', 'Payment failed with error "@error", code @code.', 
      array('@error' => check_plain($_POST['ErrorText']), '@code' => check_plain($_POST['Error'])), WATCHDOG_ERROR);
    echo 'REDIRECT=' . url('ubercart_constriv/failed/' . check_plain($_POST['Error']), array('absolute' => TRUE));
    exit();
  }

  if (!isset($_POST["paymentid"]) || !isset($_POST["tranid"])) {
    watchdog('ubercart_constriv', 'NotificationMessage attempted with invalid payment ID or transaction ID.', array(), WATCHDOG_ERROR);
    return;
  }

  if (strpos($_POST['udf1'], '-') > 0) {
    list($order_id, $cart_id) = explode('-', $_POST['udf1']);

    // Sanitize order ID and cart ID
    $order_id = intval($order_id);
    $cart_id  = check_plain($cart_id);

    if (!empty($cart_id)) {
      // Needed later by uc_complete_sale to empty the correct cart
      $_SESSION['uc_cart_id'] = $cart_id;
    }
  }
  else {
    $order_id = intval($_POST['trackid']);
  }

  watchdog('ubercart_constriv', 'Receiving NotificationMessage for order @order_id. <pre>@debug</pre>', array('@order_id' => $order_id, '@debug' => variable_get('ubercart_constriv_debug', FALSE) ? print_r($_POST, TRUE) : ''));

  $order = uc_order_load($order_id);

  if ($order == FALSE) {
    watchdog('ubercart_constriv', 'NotificationMessage attempted for non-existent order @order_id.', array('@order_id' => $order_id), WATCHDOG_ERROR);
    echo 'REDIRECT=' . url('ubercart_constriv/failed', array('absolute' => TRUE));
    return;
  }

  // Assign posted variables to local variables
  $paymentid = check_plain($_POST['paymentid']);
  $tranid = check_plain($_POST['tranid']);
  $result = check_plain($_POST['result']);
  $auth = check_plain($_POST['auth']);
  $postdate = check_plain($_POST['postdate']);
  $trackid = check_plain($_POST['trackid']);
  $ref = check_plain($_POST['ref']);
  $responsecode = check_plain($_POST['responsecode']);
  $cardtype = check_plain($_POST['cardtype']);
  $payinst = check_plain($_POST['payinst']);
  $liability = check_plain($_POST['liability']);

  $udf = array();

  for ($i = 1; $i >= 5; $i++) {
    $udf[$i] = check_plain($_POST['udf' . $i]);
  }
  
  watchdog('ubercart_constriv', 'Payment transaction verified.', array(), WATCHDOG_NOTICE);

  $duplicate = (bool) db_query_range('SELECT 1 FROM {uc_payment_tvb} WHERE txn_id = :id', 0, 1, array(':id' => $tranid))->fetchField();
  if ($duplicate) {
    watchdog('ubercart_constriv', 'Transaction ID of NotificationMessage has been processed before.', array(), WATCHDOG_NOTICE);
    return;
  }

  db_insert('uc_payment_tvb')
    ->fields(array(
      'order_id' => $order->order_id,
      'payment_id' => $paymentid,
      'txn_id' => $tranid,
      'txn_result' => $result,
      'txn_auth' => $auth,
      'txn_postdate' => $postdate,
      'txn_trackid' => $trackid,
      'txn_ref' => $ref,
      'txn_udf' => json_encode($udf),
      'txn_responsecode' => $responsecode,
      'cardtype' => $cardtype,
      'payinst' => $payinst,
      'liability' => $liability,
      'received' => REQUEST_TIME,
    ))
    ->execute();

  if ($result == 'APPROVED' || $result == 'CAPTURED') {

    $comment = t('TVB transaction ID: @txn_id<br />Merchant bank transaction ID: @ref<br />Payment ID: @paymentid<br />Card used: @card<br />Authorization code: @auth<br />Security: @sec',
      array('@txn_id' => $tranid,
            '@ref' => $ref,
            '@paymentid' => $paymentid,
            '@card' => $cardtype,
            '@auth' => $auth,
            '@sec' => $payinst,
      ));

    uc_payment_enter($order_id, 'tvb', $order->order_total, $order->uid, NULL, $comment);
    uc_cart_complete_sale($order);
    uc_order_comment_save($order->order_id, 0, t('Triveneto Bassilichi reported a payment of @amount @currency.',
      array('@amount' => uc_currency_format($order->order_total, FALSE), '@currency' => 'EUR')));

    echo 'REDIRECT=' . url('ubercart_constriv/complete/' . $paymentid, array('absolute' => TRUE));

  } else {

    watchdog('ubercart_constriv', 'Transaction for order @order has failed.', array('@order' => $order->order_id), WATCHDOG_ERROR);

    uc_order_comment_save($order->order_id, 0, t('Triveneto Bassilichi reported a failed payment with result of: @result (@reason)',
      array('@result' => _ubercart_constriv_result_message($result), '@reason' => _ubercart_constriv_responsecode_message($responsecode))), 'admin', 'canceled');

    echo 'REDIRECT=' . url('ubercart_constriv/failed/' . $paymentid, array('absolute' => TRUE));
  }
}

/**
 * Handles a complete Website Payments Standard sale.
 */
function ubercart_constriv_complete($paymentid) {

  if (variable_get('ubercart_constriv_debug', FALSE)) {
    watchdog('ubercart_constriv', 'Got PaymentID for complete payment: @paymentid', array('@paymentid' => $paymentid), WATCHDOG_DEBUG);
  }

  // If the order ID specified in the return URL is not the same as the one in
  // the user's session, we need to assume this is either a spoof or that the
  // user tried to adjust the order on this side while at TVB HPP. If it was a
  // legitimate checkout, the NotificationMessage will still come in from TVB
  // so the order gets processed correctly.
  // We'll leave an ambiguous message just in case
  $order_id = db_query('SELECT order_id FROM {uc_payment_tvb} WHERE payment_id = :payment_id',
    array(':payment_id' => $paymentid))->fetchField();

  if (!$order_id || !isset($_SESSION['cart_order']) || intval($_SESSION['cart_order']) != $order_id ||
      !isset($_SESSION['PaymentId']) || $_SESSION['PaymentId'] != $paymentid) {
    drupal_set_message(t('Thank you for your order! Triveneto Bassilichi will notify us once your payment has been processed.'));
    drupal_goto('cart');
  }

  $order = uc_order_load($order_id);

  // Ensure the payment method is TVB.
  if ($order->payment_method != 'tvb') {
    drupal_goto('cart');
  }

  // This lets us know it's a legitimate access of the complete page.
  $_SESSION['uc_checkout'][$_SESSION['cart_order']]['do_complete'] = TRUE;
  drupal_goto('cart/checkout/complete');
}

/**
 * Handles a failed payment.
 */
function ubercart_constriv_failed($paymentid = 0) {
  unset($_SESSION['cart_order']);

  if ((is_numeric($paymentid) && $paymentid == 0) || (!is_numeric($paymentid) && _ubercart_constriv_error_message($paymentid) == '')) {

    drupal_set_message(t('Payment is failed due to technical errors. Please feel free to continue shopping or contact us for assistance.'), 'error');

  } elseif (!is_numeric($paymentid)) {

    drupal_set_message(t('Your payment has failed with error "@reason". Please feel free to continue shopping or contact us for assistance.', array('@reason' => _ubercart_constriv_error_message($paymentid))), 'error');

  } else {

    $error_id = db_query('SELECT txn_responsecode FROM {uc_payment_tvb} WHERE payment_id = :payment_id', array(':payment_id' => $paymentid))->fetchField();

    drupal_set_message(t('Your payment has failed with error "@reason". Please feel free to continue shopping or contact us for assistance.', array('@reason' => _ubercart_constriv_responsecode_message($error_id))), 'error');
  }

  drupal_goto(variable_get('ubercart_constriv_cancel_return_url', 'cart'));
}
